#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#       changeDictionaryAllRegions.py
#
#           changeDictionaryAllRegionsDictの内容に従って、各regionに
#           changeDictionaryRegionsDictを作り、各regionに対し、changeDictionaryを
#           実行してdictionaryを書き換える。
#

import os
import glob
import getFoamContents
import setFoamContents

caseDir = ""
contentsDict = ""

#
#  getEachRegions
#    AllregionsDictから書き換えるregionを取得
def getEachRegions():
    global caseDir, contentsDict
    fileName = caseDir + "/system/changeDictionaryAllRegionsDict"
    f=open(fileName)
    contentsDict = f.read()
    f.close()
    p = getFoamContents.skipFoamFileMem(contentsDict, 0)
    regions = []
    while p<len(contentsDict):
        [[region, dummy], p] = getFoamContents.getNextKeywordAndContentsMem(contentsDict, p)
        if region != "":
            regions.append(region)
    return regions

#
#  setChangeDictionaryRegionsDict
#    各reginのchangeDictionaryDictを作成し、各regionにセット
def setChangeDictionaryRegionsDict(regions):
    global caseDir, contentsDict
    error = ""
    #errorチェック
    for name in regions:
        fileName = fileName = caseDir + "/system/" + name
        if len(glob.glob(fileName)) == 0:
            error = u"region「" + name + u"」が存在しません。"
            break
    if error != "":
        return error

    #foamFile作成
    foamFile = setFoamContents.makeFoamHeader()
    foamFile += setFoamContents.makeFoamFile("2.0", "ascii", "dictionary", "system", "changeDictionaryDict")
    footer = setFoamContents.makeFooter()

    #changeDictionaryDictを作成
    p=0
    for name in regions:
        [contents, p] = getFoamContents.getKeywordContentsMem(name, contentsDict, p)
        fileName = caseDir + "/system/" + name + "/changeDictionaryDict"
        f=open(fileName, "w")
        f.write(foamFile + contents + footer)
        f.close()
    return error

#
#  runEachChangeDictionary
#    各regionのchangeDictionaryを実行
def runEachChangeDictionary(regions):
    for name in regions:
        comm = "changeDictionary -region " + name
        print comm
        os.system(comm)


#
#  メイン
#

#  caseDirを取得
caseDir = os.getcwd()
#  変更するregionを取得
regions = getEachRegions()
#  changeDictionaryを作成し各regionにセット
error = setChangeDictionaryRegionsDict(regions)
#  changeDictionaryを実行
if error == "":
    runEachChangeDictionary(regions)
else:
    print error

