#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   gridEditor
#
#       grid editorを起動する
#
#       使い方
#       gridEditor.py <caseDir> <timeFolder> <polyMeshFolder>
#       例：gridEditor.py ~/cavity 0 constant/polyMesh
#
#   11/08/11    新規作成
#   12/01/21    バグ修正
#   12/03/25    起動時に読み込むfieldを設定
#

import os
import os.path
import sys
import glob
import string
import treefoam
import pyTreeFoam
import getFoamContents
import editBoundaryConditionsDialog
import selectFieldsForEditDialog
import openGridEditorDialog

nTreeFoam = "-1"        #ここから起動したgridEditorのnTeeFoamは「-1」
TreeFoamPath = ""
caseDir = ""
editDir = ""            #fieldDir
polyMeshDir = ""


#
#  showGridEditor
#
def showGridEditor(editDir, polyMeshDir, fields):
    if len(fields) == 0:
        foldersFiles = pyTreeFoam.getFoldersFiles(editDir)
        fields = foldersFiles[1]

    #Grid編集
    if len(fields) > 0:
        #title, dirを設定
        caseFolder = editDir.split("/")[-2]
        timeFolder = editDir.split("/")[-1]
        title = ": " + caseFolder + "/" + timeFolder
        caseDir = string.join(editDir.split("/")[:-2], "/")
        openGridEditorDialog.Show(nTreeFoam, title, "", "" , editDir, timeFolder)
        """
        try:
            editBoundaryConditionsDialog.Show(nTreeFoam, title, polyMeshDir,
                    editDir, fields)
        except:
            mess = u"error:オープンできないfile（field or boundary）があります。"
            print mess
        """
    else:
        mess = u"error:Fieldが選択されていません。\n  Fieldを選択してください。"
        print mess


#  getInstallFolder
#    TreeFoamのインストールpathを取得
def getInstallFolder():
    homeFolder = os.path.expanduser("~")
    fileName = homeFolder + "/.treeFoamInstallFolder"
    contents = getFileContents(fileName)[0]
    return contents

#  getFileContents
#    ファイル内容を読み込む
def getFileContents(fileName):
    f=open(fileName); contents=f.readlines(); f.close()
    return contents

#  setFileContents
#    ファイル内容を書き込む
def setFileContents(fileName, contents):
    f=open(fileName, "w")
    for line in contents:
        f.write(line)
    f.close()

#  checkDir
#    入力されたDirのチェック
def checkDir(a1, a2, a3):
    global caseDir, editDir, polyMeshDir
    error = ""
    caseDir = os.path.abspath(a1)
    editDir = os.path.abspath(a2)
    polyMeshDir = os.path.abspath(a3)
    if len(glob.glob(a1)) == 0:
        error = "指定されたcaseDirは、存在していません。\n " + caseDir
    elif len(glob.glob(a2)) == 0:
        error = "指定されたeditDirは、存在していません。\n " + editDir
    elif len(glob.glob(a3)) == 0:
        error = "指定されたpolyMeshDirは、存在しません。\n " + polyMeshDir
    if error != "":
        error = "error:" + error + "\n"
    return error

#  getTimeFolder
#    timeFolderを取得
def getTimeFolder(a1):
    caseDir = os.path.abspath(a1)
    items = getFoamContents.controlDict(caseDir+"/system", ["startFrom", "startTime"])
    if items[0] == "startTime":
        timeFolder = items[1]
    if items[0] == "latestTime":
        timeFolders = treefoam.getResultTimeFolders(caseDir)
        timeFolder = timeFolders[-1]
    return timeFolder


#  checkFields
#    fieldsのチェック
def checkFields():
    global fields, editDir
    wFields = []
    for field in fields:
        fieldDir = editDir + "/" + field
        if len(glob.glob(fieldDir)) != 0:
            wFields.append(field)
        else:
            print "「" + field + "」が存在しません。"
    if len(wFields) == 0:
        print "読み込むfieldが無かったので、全てのfieldを選択しました。"
    fields = wFields

#
#  getCommand
#    command取得
def getCommand():
    global dirCaseNum, fields
    comm = "run"
    i=1
    while i<len(sys.argv):
        val = sys.argv[i]
        if val == "-h" or val == "-help" or val == "--help":
            comm = "help"
            i = len(sys.argv)
        elif val == "-f" or val == "-fields" or val == "--fields":
            i += 1
            flag = 0
            fields = []
            while i<len(sys.argv) and flag==0:
                val = sys.argv[i]
                if val[0] != "-":
                    fields.append(val)
                    i += 1
                else:
                    flag = 1
                    i -= 1
            comm = "run"
        elif val == "-d" or val == "-directory" or val == "--directory":
            i += 1
            flag = 0
            dirCaseNum = []
            while i<len(sys.argv) and flag == 0:
                j=0
                val = sys.argv[i]
                if val[0] != "-":
                    dirCaseNum.append(val)
                    j += 1
                    i += 1
                else:
                    flag = 1
                    i -= 1
            comm = "run"
        else:
            flag = 0
            dirCaseNum = []
            while i<len(sys.argv) and flag == 0:
                j=0
                val = sys.argv[i]
                if val[0] != "-":
                    dirCaseNum.append(val)
                    j += 1
                    i += 1
                else:
                    flag = 1
                    i -= 1
            comm = "run"
        i += 1
    return comm


#
#  メイン
#  ------
#

#変数クリア
error = ""
a1=""
a2=""
a3=""
dirCaseNum = []
fields = []

#TreeFoamのインストールpath取得
TreeFoamPath = getInstallFolder()

#commnad取得
comm = getCommand()

#各コマンドの実行
if comm == "help":
    error = "使い方\n"
    error += "  gridEditor [option]\n"
    error += "  option\n"
    error += "      -d      読み込むdirectoryを設定\n"
    error += "              <caseDir> <fieldFolder> <polyMeshFolder>を設定\n"
    error += "              省略時は、currentDirがcaseDirになる。\n"
    error += "      -directory   同上\n"
    error += "      --directory  同上\n"
    error += "      -h      helpを表示\n"
    error += "      -help     同上\n"
    error += "      --help    同上\n"
    error += "\n"
    error += "  ex: gridEditor -d ~/cavity 0 constant/polyMesh\n"
    error += "      gridEditor -d ~/cavity 0\n"
    error += "      gridEditor -d ~/cavity\n"
    error += "      gridEditor ~/cavity\n"

elif comm == "run":
    if len(dirCaseNum) == 0:
        a1 = "."
        a2 = getTimeFolder(a1)
        a3 = "constant/polyMesh"
    elif len(dirCaseNum) == 1:
        a1 = dirCaseNum[0]
        a2 = getTimeFolder(a1)
        a3 = "constant/polyMesh"
    elif len(dirCaseNum) == 2:
        a1 = dirCaseNum[0]
        a2 = dirCaseNum[1]
        if len(a2.split("/")) == 2:
            folder = a2.split("/")[1]
            a3 = "constant/" + folder + "/polyMesh"
        else:
            a3 = "constant/polyMesh"
    elif len(dirCaseNum) == 3:
        a1 = dirCaseNum[0]
        a2 = dirCaseNum[1]
        a3 = dirCaseNum[2]        
    else:
        error = "error:アーギュメントの数が合いません。\n gridEditor <caseDir> <fieldFolder> <polyMeshFolder>\n Ex: gridEditor ~/cavity 0 constant/polyMesh"


if error != "":
    print "\n" + error
else:
    #dirのチェック
    error = checkDir(a1, a2, a3)
    if error != "":
        print "\n" + error
    else:
        #gridRditor起動
        showGridEditor(editDir, polyMeshDir, fields)

