#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  unv2gmshToFoamNameConvert.py
#    face名、volume名をFoam側に設定する。
#
#       09/08/02    Face名又はCell名の合計が10ヶを越える場合、名称変換が正常にできなかった
#                   事を修正。setsフォルダ内のファイル内容についても名称変換。
#       09/08/31    エラーメッセージの内容を充実。
#       10/04/15    バグ修正。同じ文字列を含んだ名前を変換する時、うまく変換できなかった
#       10/08/09    未定義のvolume名をcellZone_defaultとして定義を追加
#       12/08/05    変換後の名称を「cellZone_」→「cells_」に変更
#

import os
import sys
import glob

gmshFile = "model/salomeNameList.txt"
foamFile = "model/foamNameList.txt"
workdir = ""
salomeName = []
foamName = []; foamNameNo = []
faceZoneNo = []
cellZoneNo = []


#
#  setCurrentDirectory
#    Current directoryをworkdirにセット
#    workdir
def setCurrentDirectory():
  global workdir
  workdir = os.getcwd()


#
#  getSalomeName
#
def getSalomeName():
  global workdir,gmshFile,salomeName
  print "Salome側（unv file）のグループ名を取得..."
  salomeName=[]
  name = []; sname = []
  f = open(workdir + '/' + gmshFile)
  for a in f.readlines():
    name.append(a[2:])
  for a in name:
    i=0; b=""
    while a[i] != " ":
      b=b+a[i]
      i=i+1
    salomeName.append(b)
  f.close

#
#  getFormName
#
def getFoamName():
  global workdir,foamFile
  print "Foam側で設定したpatch名を取得..."
  name = []
  f = open(workdir + '/' + foamFile)
  for a in f.readlines():
    b = a.split()
    b[0]=int(b[0])
    name.append(b)
  f.close
  name.sort()
  i=0
  while i < len(name):
    a=(name[i])[1]
    foamName.append(a)
    i=i+1

#
#  checkSalomeFoamName
#　　未定義のvolume名を検出し、cellZone_defaultとして定義
def checkSalomeFoamName():
  global workdir, gmshFile, foamFile
  f = open(workdir + "/" + foamFile)
  foamCont = []
  for line in f.readlines():
    foamCont.append(line)
  f.close()
  a = foamCont[1].split()
  if (a[0] == "0"):                 #「0」のcellZoneがあるか？
    a = foamCont[0].split()
    b = foamCont[1].split()
    if (a[1][:4] == "cell"):        #あれば「-1」に変更
      foamCont[0] = "-1 " + a[1] + "\n"
    if (b[1][:4] == "cell"):
      foamCont[1] = "-1 " + b[1] + "\n"
    foamCont.sort()
    print "定義されていないcellZoneがあります。cells_defaultで定義しました"

    f = open(workdir + "/" + foamFile, "w")
    i=0
    while i<len(foamCont):
      f.write(foamCont[i])
      i=i+1
    f.close()

    salomeCont = []
    salomeCont.append("  default -1\n")    
    f=open(workdir + "/" + gmshFile)
    for line in f.readlines():
      salomeCont.append(line)
    f.close()
    f=open(workdir + "/" + gmshFile, "w")
    i=0
    while i<len(salomeCont):
      f.write(salomeCont[i])
      i=i+1
    f.close()

    getSalomeName()
    

#
# changePatchName
#
def changePatchName():
  global workdir, salomeName, foamName, foamNameNo
  print "boundaryファイルに設定されたpatch名をSalome側の名称に変換..."
  boundaryFile = workdir + '/constant/polyMesh/boundary'
  f=open(boundaryFile)
  contents = f.readlines()
  f.close

  i=0
  for line in contents:
    name = line.split()
    if len(name) == 1:
      s = 0
      for Fname in foamName:
        if name[0] == Fname:
          Sname = salomeName[s]
          cont = exchangeValue(Fname, Sname, line)
          contents[i] = cont
          break
        s=s+1
    i=i+1

    f = open(boundaryFile, "w")
    for line in contents:
      f.write(line)
    f.close()

def exchangeValue(name, cname,line):
  i = line.index(name)
  lineR = line[:i] + cname + "\n"
  return lineR


#
#  setCellFaceZoneName
#  
def setCellFaceZoneName():
  global workdir
  print "faceZones, cellZones, sets内ファイル名をsalome側名称に変換..."
  #faceZoneファイルの修正
  fileName = workdir + "/constant/polyMesh/faceZones"
  command = "cp " + fileName + " " + fileName + ".bak"
  os.system(command)
  f=open(fileName)
  contents = f.readlines()
  f.close()
  zoneName = []
  setsFiles = glob.glob("constant/polyMesh/sets/*")
  for name in setsFiles:
    zoneName.append(name.split("/")[-1])
  for name in zoneName:
    if name[0:4] == "face":
      No = "patch" + name[9:]
      i = 0
      for fName in foamName:
        if No == fName:
          faceZoneNo.append(i)
        i = i + 1
  command = ""
  for i in faceZoneNo:
    salomeN = "faceZone_"+salomeName[i]
    foamN = "faceZone_"+(foamName[i])[5:]
    n=0
    for line in contents:
      lineName = line.split()
      if len(lineName) != 0:
        contName = lineName[0]
        if contName == foamN:
          #contents[n] = salomeN + "\n"
          contents[n] = "faces_" + salomeName[i] + "\n"
          break
      n=n+1
    f=open(fileName, "w")
    for line in contents:
      f.write(line)
    f.close()

  #cellZoneファイルの修正
  fileName = workdir + "/constant/polyMesh/cellZones"
  command = "cp " + fileName + " " + fileName + ".bak"
  os.system(command)
  f=open(fileName)
  contents = f.readlines()
  f.close()
  i= 0
  for nameF in foamName:
    if nameF[0:4] == "cell":
      cellZoneNo.append(i)
    i=i+1
  for i in cellZoneNo:
    Oname = foamName[i]
    #Cname = "cellZone_" + salomeName[i]
    Cname = "cells_" + salomeName[i]
    n=0
    for line in contents:
      lineName = line.split()
      if len(lineName) != 0:
        contName = lineName[0]
        if contName == Oname:
          contents[n] = Cname + "\n"
          break
      n=n+1
    f=open(fileName, "w")
    for line in contents:
      f.write(line)
    f.close()

  #setsフォルダ内のファイル名修正
  fileDir = workdir + "/constant/polyMesh/sets/"
  for i in faceZoneNo:
    Oname = fileDir + "faceZone_" + (foamName[i])[5:]
    #Cname = fileDir + "faceZone_" + salomeName[i]
    Cname = fileDir + "faces_" + salomeName[i]
    command = 'cat ' + Oname + ' | sed "s/faceZone_' + (foamName[i])[5:] + '/faceZone_' + salomeName[i] + '/" > ' + Cname
    os.system(command)
    command = "rm " + Oname
    os.system(command)

  for i in cellZoneNo:
    Oname = fileDir + foamName[i]
    #Cname = fileDir + "cellZone_" + salomeName[i]
    Cname = fileDir + "cells_" + salomeName[i]
    #command = 'cat ' + Oname + ' | sed "s/cellZone_' + (foamName[i])[9:] + '/cellZone_' + salomeName[i] + '/" > ' + Cname
    command = 'cat ' + Oname + ' | sed "s/cells_' + (foamName[i])[9:] + '/cells_' + salomeName[i] + '/" > ' + Cname
    os.system(command)
    command = "rm " + Oname
    os.system(command)

 
#
#  writeConvertMap
#    グループ名変換結果の書き出し
#
def writeConvertMap():
  global wordir
  writeFileName = workdir + "/model/map.map"
  f = open(writeFileName, "w")
  cont = "\nSalome 側のグループ名の変換結果\n\n"
  i = 0
  for name in salomeName:
    cont = cont + name + "\t" + str(i) + "\t" + foamName[i] +"\n"
    i = i + 1
  #cont = cont + "\nfaceZone\n"
  cont = cont + "\nfaces\n"
  for n in faceZoneNo:
    #line = "    " + foamName[n] + "  faceZone_" + salomeName[n] + "\n"
    line = "    " + foamName[n] + "  faces_" + salomeName[n] + "\n"
    cont = cont + line
  #cont = cont + "\ncellZone\n"
  cont = cont + "\ncells\n"
  for n in cellZoneNo:
    #line = "    " + foamName[n] + "  cellZone_" + salomeName[n] + "\n"
    line = "    " + foamName[n] + "  cells_" + salomeName[n] + "\n"
    cont = cont + line
  f.write(cont)
  f.close
  


#
# メイン
#
setCurrentDirectory()
getSalomeName()
getFoamName()
checkSalomeFoamName()       #volumeの定義をチェック

if len(salomeName) == len(foamName):
  changePatchName()
  setCellFaceZoneName()
  writeConvertMap()
else:
  f = open("./model/error.txt","w")
  message = "unv2gmshToFoam 変換エラー\n"
  message = message + "Salome側のグループ数と変換後のFoam側のグループ数が合いません。\n"
  message = message + "（標準出力の出力結果でsalomeとfoamのグループ名を確認してください。）"
  message = message + "Salome側でVolumeをグループ化していない可能性があります。\n"
  message = message + "Patch名のみグループ化している場合、「ideasUnvToFoam」を使って変換してください。\n"
  message = message + "faceがオーバラップして定義されている、または同じ面を違う名前で定義した場合は、エラーが発生します。"
  f.write(message)
  f.close
  print "\nSalome側のグループ数と変換後のFoam側のグループ数が合いません。"
  print "Salomeグループ名:"
  print salomeName
  print "Foamグループ名:"
  print foamName
  print

