#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   module of OFLauncher.py
#
#       10/01/04    新規作成
#          02/16    「checkCase」を「0」フォルダのチェックから「数字フォルダ」の
#                   チェックに変更
#          02/21    unv2gmshToFoam変換のバグ修正
#          08/09    unv2gmshToFoam変換前に、setsフォルダ内をクリアするように変更
#

import os
import glob
import dircache
import OKDialog
import ScaleOKDialog
import MakePatchDialog
import clearBoundaryFieldDialog
import SolveOKDialog
import selectFieldsDialog
import getFoamContents
import setFoamContents
import selectTimeFolderDialog
import selectFolderFilesDialog
import selectPropertiesFilesDialog
import mergeRegionsCellZoneDialog
import splitMeshCellZones
import selectCellZoneLSDialog
import calcMagForceDialog
import calcEmForceDialog
import createNewCaseDialog
import changeSolverDialog
import SolveParaOKDialog
import pyTreeFoam
#import mergeZonesDialog

folderNo = ""
magFile = "magnification"           #モデルの倍率


def createNewCase():
    dataFile = "/home/caeuser/bin/solverList.txt"
    solverCase = readTutorialsCaseData(dataFile)
    createNewCaseDialog.Show(solverCase)


def changeSolver():
    if checkCase() == "OK":
        dataFile = "/home/caeuser/bin/solverList.txt"
        solverCase = readTutorialsCaseData(dataFile)
        changeSolverDialog.Show(solverCase)
    else:
        title = "Case Name"
        message = u"現在のDirは、OpenFOAMのCaseではありません"
        OKDialog.Show(title, "error", message)


def readTutorialsCaseData(fileName):
    f=open(fileName)
    cont=f.readlines()
    f.close()
    solverCase = []
    i=1
    while i < len(cont):
        cont[i] = cont[i].decode("utf-8")
        data = cont[i].split()
        if len(data) == 3:
            data.append("")
        solverCase.append(data)
        i=i+1
    return solverCase



def ideasUnvToFoam():
    files = glob.glob("model/mesh.unv")
    if len(files) == 0:
        title = "ideasUnvToFoam 変換エラー"
        mess = u"「model/mesh.unv」がありません。"
        img = "error"
        OKDialog.Show(title, img, mess)
        return
    #command = '. /home/caeuser/bin/bashrc-FOAM;ideasUnvToFoam ./model/mesh.unv 2> ./model/error.txt | tee ./model/meshconv.map'
    command = 'ideasUnvToFoam ./model/mesh.unv 2> ./model/error.txt | tee ./model/meshconv.map'
    #os.system(command)
    pyTreeFoam.runOFUtility(command)
    n=[]
    f = open("./model/error.txt")
    for line in f.readlines():
        n.append(line)
    f.close
    if len(n) != 0:
        mess ="\n変換エラーが発生しました。\n  エラーの内容は、「./model/error.txt」を確認してください。\n"
        print mess
        title = "ideasUnvToFoam 変換エラー"
        img = "error"
        OKDialog.Show(title, img, mess)
    else:
        clearInternalBoundary()
        setMagnification("1.0")           #モデルの倍率をセット


def makeMagFile(mag):
    wdir = os.getcwd()
    magFile = wdir + "/model/magnification"
    cont = u"#\n#  model scale\n#\n\n" + mag + "\n"
    f=open(magFile, "w")
    f.write(cont)
    f.close()


def unv2gmshToFoam():
    files = glob.glob("model/mesh.unv")
    if len(files) == 0:
        title = "unv2gmshToFoam 変換エラー"
        mess = u"「model/mesh.unv」がありません。"
        img = "error"
        OKDialog.Show(title, img, mess)
        return
    unvFile = ' ./model/mesh.unv'
    gmshFile = ' ./model/meshconv'
    gmshLogFile = ' ./model/meshconv.map.map'
    command = "rm ./constant/polyMesh/sets/*"       #setsフォルダ内をクリア
    os.system(command)
    command = 'python ~/TreeFoam/bin/unv2gmsh.py' + unvFile + gmshFile
    os.system(command)
    command = 'gmshToFoam' + gmshFile + '.msh | tee ' + gmshLogFile
    pyTreeFoam.runOFUtility(command)
    print "unv2gmshToFoam メッシュ変換は、完了しました。"
    print "<unv2gmshToFoamSmooth>を実行します。"
    command = "unv2gmshToFoamSmooth"
    pyTreeFoam.runOFUtility(command)
    n = []      #エラーチェック
    f = open("./model/error.txt")
    for line in f.readlines():
        n.append(line)
    f.close
    if len(n) != 0:
        eFlag = 1
        title = "unv2gmshToFoam変換\n"
        message = "変換エラーが発生しました。"
        for line in n:
            message = message + line
        img = "error"
        OKDialog.Show(title, img, message)
    else:
        clearInternalBoundary()
        setMagnification("1.0")           #モデルの倍率をセット


def setMagnification(scale):
    global magFile
    cont = []
    cont.append("#\n")
    cont.append("#  モデルの倍率の記録\n")
    cont.append("#\n")
    cont.append("\n")
    cont.append(scale + "\n")
    fileName = "./model/" + magFile
    f=open(magFile, "w"); f.write(cont); f.close() 


def clearInternalBoundary():
    print "<clearInternalFields.py>を実行します。"
    command = 'clearInternalFields.py'
    pyTreeFoam.runOFUtility(command)
    #print "<clearBoundaryFields.py>を実行します。"
    #command = 'clearBoundaryFields.py'
    #pyTreeFoam.runOFUtility(command)
    #print "\n変換処理は、終了しました。"
    makeMagFile("1.0")                      #モデルの倍率を記録
    message = "unv2gmshToFoam の処理が終了しました。\n  引き続きモデルのスケールを変更しますか？"
    XYZ = getModelSize()
    modelSize = str(XYZ[0]) + ' x ' + str(XYZ[1]) + ' x ' + str(XYZ[2])
    print "modelSize =", modelSize        
    ScaleOKDialog.Show(message, modelSize)



def scaleMesh():
    # モデルの大きさをセット
    XYZ = getModelSize()
    modelSize = str(XYZ[0]) + ' x ' + str(XYZ[1]) + ' x ' + str(XYZ[2])
    message = "モデルのスケールを変更します。"
    ScaleOKDialog.Show(message, modelSize)

# モデルのサイズを取得
def getModelSize():
    CaseDir = os.getcwd()
    f = open(CaseDir + '/constant/polyMesh/points')
    line = f.readline()
    while line[0]!="(":
      line = f.readline()
    line = f.readline()
    line = line[1:-2]
    XYZ = line.split()
    maxX = float(XYZ[0])
    minX = maxX
    maxY = float(XYZ[1])
    minY = maxY
    maxZ = float(XYZ[2])
    minZ = maxZ
    line = f.readline()
    while line[0] != ")":
        line = line[1:-2]
        XYZ = line.split()
        X = float(XYZ[0])
        Y = float(XYZ[1])
        Z = float(XYZ[2])
        if maxX < X :
            maxX = X
        if minX > X :
            minX = X
        if maxY < Y:
            maxY = Y
        if minY > Y:
            minY = Y
        if maxZ < Z:
            maxZ = Z
        if minZ > Z:
            minZ = Z
        line = f.readline()
    f.close
    XYZ[0] = abs(maxX-minX)
    XYZ[1] = abs(maxY-minY)
    XYZ[2] = abs(maxZ-minZ)
    return XYZ



def makePatch():
    patches = getInternalPatchName()
    MakePatchDialog.Show(patches)

def getInternalPatchName():
    ans = []
    files = glob.glob("./constant/polyMesh/sets/*")
    faceSets = []
    for name in files:
        fileName = name.split("/")[-1]
        if (fileName[-1] != "~"
          and fileName != "faceZone_Default"
          and fileName != "Default"):
            fileClass = getFoamFileClass("./constant/polyMesh/sets", fileName)
            if fileClass == "faceSet":
                faceSets.append(fileName)
    return faceSets

def getFoamFileClass(wdir, name):
    items = getFoamContents.getFoamFile(wdir, name)
    for item in items:
        if item[0] == "class":
            fileClass = item[1]
    return fileClass


def splitRegionsCellZone():
    error = splitMeshCellZones.Run()
    if error[0] == "NG":
        OKDialog.Show("エラー", "error", error[1])
    else:
        msg = "領域（region）に分割しました。"
#        OKDialog.Show("領域分割　終了", "info", msg)
#       引き続きchtMultiRegionFoamで解析する場合は、以下を実行。
        saveDir = os.getcwd()
        regDir = saveDir + "/regCase"
        os.chdir(regDir)
        wdir = regDir + "/constant/polyMesh"
        cellZones = getFoamContents.cellZones(wdir)
        zoneNames =[]
        i=0
        while i < len(cellZones):
            zoneNames.append(cellZones[i][0])
            i=i+1
        job = "mergeRegionsCellZone"
        zoneNames.sort()
        selectCellZoneLSDialog.Show(job, zoneNames)


def clearBoundaryFields():
    job = "clearBoundary"
    # 時間フォルダを取得
    wdir = os.getcwd()
    results = getTimeFolders(wdir)
    if len(results) <= 0:
        # error 処理
        print "error メッセージ"
        wdir = os.getcwd()
        title = "error"
        img = "error"
        msg = "現在のフォルダ\n  " + wdir + "\nは、CaseDirでは、ありません。"
        OKDialog.Show(title, img, msg)
    else:
        # 時間フォルダを選択する
        items = []
        items.append("startFrom")
        items.append("startTime")
        itemsValue = getFoamContents.controlDict(wdir+"/system", items)
        if itemsValue[0] == "latestTime":
            subMess = items[0] + " " + itemsValue[0]
            selection = len(results)-1
        else:
            subMess = items[0] + " " + itemsValue[0] + "; " + itemsValue[1]
            selection = 0
            i=0
            while  i < len(results):
                if itemsValue[1] == results[i]:
                    selection = i
                    i = len(results)
                i = i + 1
        # 時間フォルダを選択するDialogを起動
        selectTimeFolderDialog.Show(job, subMess, results, selection)



def clearInternalFields():
    job = "clearInternal"
    # 時間フォルダを取得
    wdir = os.getcwd()
    results = getTimeFolders(wdir)
    if len(results) <= 0:
        # error 処理
        print "error メッセージ"
        wdir = os.getcwd()
        title = "error"
        img = "error"
        msg = "現在のフォルダ\n  " + wdir + "\nは、CaseDirでは、ありません。"
        OKDialog.Show(title, img, msg)
    else:
        # 時間フォルダを選択する
        items = []
        items.append("startFrom")
        items.append("startTime")
        itemsValue = getFoamContents.controlDict(wdir+"/system", items)
        if itemsValue[0] == "latestTime":
            subMess = items[0] + " " + itemsValue[0]
            selection = len(results)-1
        else:
            subMess = items[0] + " " + itemsValue[0] + "; " + itemsValue[1]
            selection = 0
            i=0
            while  i < len(results):
                if itemsValue[1] == results[i]:
                    selection = i
                    i = len(results)
                i = i + 1
        # 時間フォルダを選択するDialogを起動
        selectTimeFolderDialog.Show(job, subMess, results, selection)

def makeFieldsCellzones():
    cont = "nameList\n\n"
    # field名取得
    cont = cont + "<Field>"
    wdir = os.getcwd()
    times = getTimeFolders(wdir)
    items = getFoamContents.controlDict(wdir+"/system", ["startFrom"])
    if items[0] == "startTime":
        allFiles = getFoldersFiles(wdir + "/" + times[0])
        cont = cont + " startTime " + times[0] + "\n"
    elif items[0] == "latestTime":
        allFiles = getFoldersFiles(wdir + "/" + times[-1])
        cont = cont + " latestTime " + times[-1] + "\n"
    fields = allFiles[1]
    fields = delBackupFile(fields)
    fields.sort()
    cont = makeNextCont(fields, cont)
    # setsファイル取得
    files = glob.glob(wdir + "/constant/polyMesh/sets/*")
    sets = []
    for name in files:
        sets.append(name.split("/")[-1])
    sets = delBackupFile(sets)
    sets.sort()
    cont = cont + "\n<sets>\n"
    cont = makeNextCont(sets, cont)
    # cellZoneName取得
    cellZones = getFoamContents.cellZoneNames()
    cellZones = delBackupFile(cellZones)
    cellZones.sort()
    cont = cont + "\n<cellZones>\n"
    cont = makeNextCont(cellZones, cont)
    # field sets cellZone のリスト作成
    fileName = "nameList"
    f=open(fileName, "w")
    f.write(cont)
    f.close()

def delBackupFile(files):
    names = []
    for name in files:
        if name[-1] != "~":
            names.append(name)
    return names

def makeNextCont(items, cont):
    for item in items:
        cont = cont + item + "\n"
    return cont


def editPropertiesFiles():
    wdir = os.getcwd() + "/constant"
    allFiles = getFoldersFiles(wdir)
    folders = allFiles[0]
    files = allFiles[1]
    if len(folders) >= 1:
        FFlist = getFolderFilesListProperties(wdir)
        title = u"...Properties File の編集"
        img = "oneFile"
        mess = u"編集する Properties file を選択してください。\n\nProperties File は、constant フォルダ内にあります。"
        editDir = wdir
        job = "editProperties"
        # properties File の選択と編集
        selectFolderFilesDialog.Show(job, title, img, mess, FFlist, editDir)
    else:
        print "constantフォルダ内にpolyMeshフォルダがありません!\n  current directoryを確認してください。"

def getFolderFilesListProperties(wdir):
    allFiles = getFoldersFiles(wdir)
    folders = allFiles[0]; files = allFiles[1]
    FFlist = []; Flist = []
    Flist.append(".")
    for name in files:
        if name[-1] != "~":
            Flist.append(name)
    FFlist.append(Flist)
    for folder in folders:
        Flist = []
        if folder != "polyMesh":
            Flist.append(folder)
            allFiles = getFoldersFiles(wdir + "/" + folder)
            files = allFiles[1]
            for name in files:
                if name[-1] != "~":
                    Flist.append(name)
            FFlist.append(Flist)
    return FFlist



def editFvFiles():
    wdir = os.getcwd()
    region = glob.glob(wdir + "/constant/cellToRegion")
    if len(region) == 0:
        pyTreeFoam.runEditor("system/fvSchemes system/fvSolution")
        """
        command = "gedit --new-window system/fvSchemes system/fvSolution &"
        os.system(command)
        """
    else:
        allFiles = getFoldersFiles(wdir + "/system")
        folders = allFiles[0]
        files = []
        FFlist = []
        files.append(".")
        for name in allFiles[1]:
            if name == "fvSchemes" or name == "fvSolution":
                files.append(name)
        FFlist.append(files)
        for folder in folders:
            files = []
            files.append(folder)
            allFiles = getFoldersFiles(wdir + "/system/" + folder)
            for name in allFiles[1]:
                if name == "fvSchemes" or name == "fvSolution":
                    files.append(name)
            FFlist.append(files)
        title = u"計算制御 File の編集"
        img = "oneFaile"
        mess = u"メッシュが領域分割（Region）されています。\n   編集する計算制御 file を選択してください。"
        editDir = wdir + "/system"
        job = "editFvFile"
        # boundary File の選択と編集
        selectFolderFilesDialog.Show(job, title, img, mess, FFlist, editDir)


def editDict():
    wdir = os.getcwd() + "/system"
    allFiles = getFoldersFiles(wdir)
    folders = allFiles[0]; files = allFiles[1]
    Flist = []
    FFlist = []
    Flist.append(".")
    for name in files:
        if not (name[-1] == "~"): #or
                 #name == "fvSchemes" or 
                 #name == "fvSolution" or
                 #name == "controlDict" or
                 #name == "setFieldsDict"):
            Flist.append(name)
    FFlist.append(Flist)
    for folder in folders:
        Flist = []
        Flist.append(folder)
        allFiles = getFoldersFiles(wdir + "/" + folder)
        for name in allFiles[1]:
            if not (name[-1] == "~"): #or 
                     #name == "fvSchemes" or 
                     #name == "fvSolution" or
                     #name == "controlDict" or
                     #name == "setFieldsDict"):
                Flist.append(name)
        FFlist.append(Flist)
    title = u"Dict File の編集"
    img = "oneFaile"
    mess = u"編集するDict file を選択してください。\n\nDict file は、systemフォルダ内にあります。"
    editDir = wdir
    job = "editDictFile"
    # boundary File の選択と編集
    selectFolderFilesDialog.Show(job, title, img, mess, FFlist, editDir)



def checkCase():
    files = glob.glob("*")
    dir0 = ""; dirConstant = ""; dirSystem = ""
    for name in files:
        try:
            a=float(name)
            dir0 = name
        except:
            a=name
        if name == "constant":
            dirConstant = name
        if name == "system":
            dirSystem = name
    if len(dir0) >0 and len(dirConstant) > 0 and len(dirSystem) >0 :
        return "OK"
    else:
        return "NG"


def selectSolve():
    if checkCase() == "OK":
        items = []
        items.append("startFrom")
        items.append("startTime")
        items.append("endTime")
        wdir = os.getcwd()
        itemsValue = getFoamContents.controlDict(wdir+"/system", items)
        SolveOKDialog.Show()
    else:
        title = "Case Name"
        message = u"現在のDirは、OpenFOAMのCaseではありません"
        OKDialog.Show(title, "error", message)


def showConnectedNodes():
    if len(glob.glob("machines")) == 0:
        title = "nodeの表示"
        img = "error"
        message = "current directory内に「mashines」ファイルがありません。\n「machines」で並列計算させるノードを設定してください。"
        OKDialog.Show(title, img, message)
        return
    tempFile = "nodes.temp"
    command = ". bashrc-FOAM; lamboot -v machines; lamnodes | tee " + tempFile + "; lamhalt -d"
    os.system(command)
    f=open(tempFile); cont = f.read(); f.close()
    message = "接続されているノードを表示します。\n\nnode\t内容\n-------------------------------------\n" + cont
    os.system("rm -rf " + tempFile)
    title = "nodeの表示"
    img = "info"
    OKDialog.Show(title, img, message)


def runMPI():
    if checkCase() == "OK":
        if len(glob.glob("processor*")) >= 2:
            SolveParaOKDialog.Show()
        else:
            title = "並列計算実行"
            message = "meshが分割されていません!!!"
            OKDialog.Show(title, "error", message)
    else:
        title = "並列計算"
        message = "現在のDirは、OpenFOAMのCaseではありません。"
        OKDialog.Show(title, "error", message)


def runParaFoam():
    wdir = os.getcwd()
    region = glob.glob(wdir + "/constant/cellToRegion")
    if len(region) == 1:
        # Regionがある場合、読み込みファイルを作成
        timeFolders = getTimeFolders(wdir)
        allFiles = getFoldersFiles(wdir + "/" + timeFolders[0])
        folders = allFiles[0]
        command = ". bashrc-FOAM\n"
        for folder in folders:
            command = command + "paraFoam -touch -region " + folder + "\n"
        os.system(command)
    # paraFoam を起動
    command = ". bashrc-FOAM; paraFoam & "
    os.system(command)


def calcMagForce():
    wdir = os.getcwd()
    times = getTimeFolders(wdir)
    setsDir = wdir + "/constant/polyMesh/sets"
    files = glob.glob(setsDir + "/faceZone*")
    faceZones = []
    for fileName in files:
        faceZones.append(fileName.split("/")[-1])
    faceZones.sort()
    calcMagForceDialog.Show(times,faceZones)


def calcEmForce():
    wdir = os.getcwd()
    times = getTimeFolders(wdir)
    cont = getFoamContents.cellZones(wdir + "/constant/polyMesh")
    cellZones = []
    i=0
    while i < len(cont):
        cellZones.append(cont[i][0])
        i=i+1
    cellZones.sort()
    calcEmForceDialog.Show(times, cellZones)


def editBoundaryFile():
    wdir = os.getcwd()
    files = glob.glob("./constant/cellToRegion")
    if len(files) == 0:
        # 通常のメッシュの場合
        editName = "./constant/polyMesh/boundary"
        pyTreeFoam.runEditor(editName)
        """
        command = "gedit --new-window " + editName
        os.system(command)
        """
    else:
        # メッシュの領域を分割している場合
        wdir = os.getcwd()
        timeFolders = getTimeFolders(wdir)
        wdir = wdir + "/" + timeFolders[0]         #最小の時間フォルダを選択
        allFiles = getFoldersFiles(wdir)
        folders = allFiles[0]
        FFlist = []
        files = []
        files.append(".")
        files.append("boundary")
        FFlist.append(files)
        for folder in folders:
            files = []
            files.append(folder)
            files.append("boundary")
            FFlist.append(files)
        title = u"Boundary File の編集"
        img = "oneFaile"
        mess = u"メッシュが領域分割（Region）されています。\n   編集する boundary file を選択してください。\n\nRegionのファイルは、時間フォルダ「" + timeFolders[0] + u"」内にあります。\nフォルダが確認できない場合は、領域分割コマンド「splitMeshReginFoam」実行時の\n「時間フォルダ」を確認してください。"
        editDir = wdir
        job = "editBoundary"
        # boundary File の選択と編集
        selectFolderFilesDialog.Show(job, title, img, mess, FFlist, editDir)



def editFields():
    # 時間フォルダを取得
    wdir = os.getcwd()
    results = getTimeFolders(wdir)
    if len(results) > 0:
        # 時間フォルダを選択する
        items = []
        items.append("startFrom")
        items.append("startTime")
        itemsValue = getFoamContents.controlDict(wdir+"/system", items)
        if itemsValue[0] == "latestTime":
            subMess = items[0] + " " + itemsValue[0] + "; " + results[-1]
            selection = len(results)-1
        else:
            subMess = items[0] + " " + itemsValue[0] + "; " + itemsValue[1]
            selection = 0
            i=0
            while  i < len(results):
                if itemsValue[1] == results[i]:
                    selection = i
                    i = len(results)
                i = i + 1
        # 時間フォルダを選択するDialogを起動
        job = "editField"
        selectTimeFolderDialog.Show(job, subMess, results, selection)
    else:
        print "error メッセージ"
        wdir = os.getcwd()
        title = "error"
        img = "error"
        msg = "現在のフォルダ\n  " + wdir + "\nは、CaseDirでは、ありません。"
        OKDialog.Show(title, img, msg)

# Time Folder名を返す。（数字でsortする。）
def getTimeFolders(wdir):
    allFiles = getFoldersFiles(wdir)
    folders = allFiles[0]
    results = []
    for name in folders:
        try:
            a = float(name)
            results.append(name)
        except:
            a=""
    temp=[]
    for valStr in results:
        a=[]
        a.append(float(valStr))
        a.append(valStr)
        temp.append(a)
    temp.sort()
    results = []
    for time in temp:
        results.append(time[1])
    return results
   
def getFoldersFiles(wdir):      #wdirは、絶対path
    dircache.reset()
    a = dircache.listdir(wdir)
    dircache.annotate(wdir, a)
    files = []; folders = []
    for name in a:
        if name[-1] == "/":
            folders.append(name[:-1])
        else:
            if name != "":
                files.append(name)
    ans = []
    ans.append(folders)
    ans.append(files)
    return ans


#---------Utility----------------------

def readCurrentDirectory():
    f = open('/home/caeuser/bin/WorkDir.txt')
    workdir = f.readline()
    workdir = workdir[:-1]
    f.close
    if os.path.isdir(workdir):
        os.chdir(workdir)
        return workdir
    else:
        workdir = os.getcwd()
        setCurrentDirectory(workdir)
        print "保存されていた CaseDir が存在しなかったので、currDir を CaseDir に設定しました。"
        return workdir

def setCurrentDirectory(wdir):
    a = []
    f = open('/home/caeuser/bin/WorkDir.txt')
    for line in f.readlines():
        a.append(line)
    f.close
    a[0] = wdir + "\n"
    f = open('/home/caeuser/bin/WorkDir.txt',"w")
    for line in a:
        f.write(line.encode("utf-8"))
    f.close
    os.chdir(wdir)

def getName(name,contents):
    i=0
    while i < len(contents):
        line = contents[i]
        if line[0:len(name)] == name:
            a = line.split()
            ans = a[1][:-1]
            i=len(contents)
        i=i+1
    return ans

def get5items(contents):
    a=[]
    a.append("application")
    a.append("startTime")
    a.append("endTime")
    a.append("deltaT")
    a.append("writeInterval")
    a.append("startFrom")
    a.append("stopAt")
    a.append("writeControl")
    b=[]
    for name in a:
        ans = getName(name, contents)
        b.append(ans)
    val = []
    val.append(a)
    val.append(b)
    return val

def getControlDict(CaseName):
    nameList=[]
    try:
        f = open(CaseName + "/system/controlDict")
        contents = f.readlines()
        f.close()
        nameList = get5items(contents)
    except:
        print "error:OpenFOAMのCaseDirではありません!!"
        i=0; a=[]; b=[]
        while i < 8:
            a.append("")
            b.append("")
            i=i+1
        nameList.append(a)
        nameList.append(b)
    return nameList

#  delResultWithoutStartTime
#    Case内の計算結果を削除（startTime以下を除く）
def delResultWithoutStartTime():
    wdir = os.getcwd()
    folder = getTimeFolders(wdir)
    startTime = getFoamContents.controlDict(wdir+"/system", ["startTime"])[0]
    if len(folder) > 1:
        i=0
        while i < len(folder):
            if float(folder[i]) > float(startTime):
                command = "rm -rf " + folder[i]
                print command
                os.system(command)
            i=i+1

#  delResult
#    Case内の計算結果を削除
def delResult():
    wdir = os.getcwd()
    print wdir
    folder = getTimeFolders(wdir)
    if len(folder) > 1:
        i=1
        while i < len(folder):
            command = "rm -rf " + folder[i]
            print command
            os.system(command)
            i=i+1

#  delParaResultWithoutStartTime
#    並列計算結果を削除（startTime以下を除く）
def delParaResultWithoutStartTime():
    n = len(glob.glob("processor*"))
    caseDir = os.getcwd()
    while n > 0:
        cpuName = str(n-1)
        wdir = caseDir + "/processor" + cpuName
        os.chdir(wdir)
        delResultWithStartTime()
        n=n-1
    os.chdir(caseDir)

#  delParaResult
#    並列計算結果を削除
def delParaResult():
    n = len(glob.glob("processor*"))
    caseDir = os.getcwd()
    while n > 0:
        cpuName = str(n-1)
        wdir = caseDir + "/processor" + cpuName
        os.chdir(wdir)
        delResult()
        n=n-1
    os.chdir(caseDir)

#
#  delProcessor
#    メッシュ分割結果（計算結果含む）を削除
def delProcessor():
    n = len(glob.glob("processor*"))
    caseDir = os.getcwd()
    while n > 0:
        cpuName = str(n-1)
        command = "rm -rf processor" + cpuName
        print command
        os.system(command)
        n=n-1
    
    

