#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  setupDictProperties.py
#    solverに応じて必要なファイル（Dictなど）をtutorialsからコピーする。
#       使い方: setupDictProperties <solver名> [solverに応じたoption]
#

import os
import sys
import glob
import pyOFLauncher

#
#  getCaseDir
#    caseDirを取得する。
def getCaseDir(solverName, caseName):
    tempFile = "getCaseDir-test.txt"
    command = ". bashrc-FOAM; find $FOAM_RUN -name " + solverName + "> " + tempFile
    os.system(command)
    f=open(tempFile)
    wdir = f.readline()[:-1]
    f.close()
    command = 'find "' + wdir + '" -name ' + caseName + "> " + tempFile
    os.system(command)
    f=open(tempFile)
    wdir = f.readline()[:-1]
    f.close()
    command = "rm " + tempFile
    os.system(command)
    return wdir

def writeLiquidProperties(caseDir, fileNames, liquidName):
    for nameDir in liquidName:
        wdir = caseDir + "/constant/" + nameDir
        for fileName in fileNames:
            a=fileName.split("/")
            writeName = wdir + "/" + a[-1]
            writeFile(fileName, writeName)

def writeLiquidDict(caseDir, liquidDict, liquidName):
    for dictName in liquidName:
        wdir = caseDir + "/system/" + dictName
        for readName in liquidDict:
            a=readName.split("/")
            writeName = wdir + "/" + a[-1]
            writeFile(readName, writeName)
    
def writeProperties(caseDir, fileNames):
    wdir = caseDir + "/constant"
    for fileName in fileNames:
        a=fileName.split("/")
        writeName = wdir + "/" + a[-1]
        writeFile(fileName, writeName)

def writeDictFiles(caseDir, fileNames):
    wdir = caseDir + "/system"
    for fileName in fileNames:
        a=fileName.split("/")
        writeName = wdir + "/" + a[-1]
        writeFile(fileName, writeName)


def writeFile(fileName, writeName):
    f=open(fileName)
    cont = f.read()
    f.close()
    f=open(writeName, "w")
    f.write(cont)
    f.close()

#
#  checkWriteFolder
#
def checkWriteFolder(caseDir, liquid, solid):
    for name in liquid:
        folderName = caseDir + "/constant/" + name
        if len(glob.glob(folderName)) == 0:
            os.mkdir(folderName)
        folderName = caseDir + "/system/" + name
        if len(glob.glob(folderName)) == 0:
            os.mkdir(folderName)
    for name in solid:
        folderName = caseDir + "/system/" + name
        if len(glob.glob(folderName)) == 0:
            os.mkdir(folderName)

#
#  getChtMultiRegionFoamFiles
#
def getChtMultiRegionFoamFiles(tuDir, caseDir, liquidName, solidName):
    #読込み
    liquidProperties = glob.glob(tuDir + "/constant/topAir/*")
    Properties = glob.glob(tuDir + "/constant/*Properties")
    liquidDict = glob.glob(tuDir + "/system/topAir/*Dict")
    solidDict = glob.glob(tuDir + "/system/heater/*Dict")
    dictFiles = glob.glob(tuDir + "/system/*Dict")
    liquidFvFiles = glob.glob(tuDir + "/system/topAir/fv*")
    solidFvFiles = glob.glob(tuDir + "/system/heater/fv*")
    #folderのチェック
    checkWriteFolder(caseDir, liquidName, solidName)
    #書込
    writeLiquidProperties(caseDir, liquidProperties, liquidName)
    writeProperties(caseDir, Properties)
    writeLiquidDict(caseDir, liquidDict, liquidName)
    writeLiquidDict(caseDir, solidDict, solidName)        
    writeDictFiles(caseDir, dictFiles)
    writeLiquidDict(caseDir, liquidFvFiles, liquidName)
    writeLiquidDict(caseDir, solidFvFiles, solidName)
    #不要なFieldを削除
    times = pyOFLauncher.getTimeFolders(caseDir)
    delNames = ["K", "cp", "rho"]
    for name in liquidName:
        for delName in delNames:
            fileName = caseDir + "/" + times[0] + "/" + name + "/" + delName
            os.system("rm " + fileName)
    delNames = ["U", "alphat", "epsilon", "k", "p"]
    for name in solidName:
        for delName in delNames:
            fileName = caseDir + "/" + times[0] + "/" + name + "/" + delName
            os.system("rm " + fileName)

#
#  printHelp
#
def printHelp():
    print "\n<説明>\nsolverに応じて必要なファイル（Dicなど）をtutorialsからコピーする。"
    print "<使い方>\n setupDictProperties <solver名> [solverに応じたoption]"
    print

#
#  chtMultiRegionFoam
#
def chtMultiRegionFoam(comm):
    liquid = []; solid = []
    i=2
    while i < len(comm):
        if comm[i] == "-fluid":
            i=i+1
            while i < len(comm) and comm[i][0] != "-":
                liquid.append(comm[i])
                i=i+1
        elif comm[i] == "-solid":
            i=i+1
            while i < len(comm) and comm[i][0] != "-":
                solid.append(comm[i])
                i=i+1
        else:
            printHelp()
            i=len(comm)
    if len(liquid) != 0 and len(solid) != 0:
        tuDir = getCaseDir("chtMultiRegionFoam", "snappyMultiRegionHeater")
        caseDir = os.getcwd()
        getChtMultiRegionFoamFiles(tuDir, caseDir, liquid, solid)
        setupRegionProperties(caseDir,liquid, solid)
        print "Dictファイル、Propertiesファイルsetupしました。"
        error = ["OK","OK"]
    else:
        msg = u"error: fluid or solid 名が正しく認識できませんでした。\nfluid, solidともcellZone名を全て入力してください。\n"
        print msg
        error = ["NG", msg]
    return error        

#
#  setupRegionProperties
#
def setupRegionProperties(caseDir, liquid, solid):
    fileName = caseDir + "/constant/regionProperties"
    f = open(fileName); contents = f.readlines(); f.close()
    n=0
    while n < len(contents):
        try:
            i = contents[n].index("fluidRegionNames")
            if i==0:
                fluid = ""
                for name in liquid:
                    fluid = fluid + name + " "
                contents[n] = "fluidRegionNames ( " + fluid + ");\n"
            n=len(contents)
        except:
            n=n+1
    n=0
    while n < len(contents):
        try:
            i = contents[n].index("solidRegionNames")
            if i==0:
                names = ""
                for name in solid:
                    names = names + name + " "
                contents[n] = "solidRegionNames ( " + names + ");\n"
            n=len(contents)
        except:
            n=n+1
    f=open(fileName,"w")
    for line in contents:
        f.write(line)
    f.close()


#
# Run
#   引数: [<command>, <solver>, <option>, ...]
#   戻値: [<"OK" or "NG">, <message>]
def Run(comm):
    print comm
    if comm[1] == "--help" or comm[1] == "-h":
        printHelp()
        error = ["OK", ""]
    elif comm[1] == "chtMultiRegionFoam":
        error = chtMultiRegionFoam(comm)
    else:
        msg = "error: " + comm[1] + " のsolverは、準備されていません。"
        error = ["NG", msg]
        print msg
    return error


if __name__ == "__main__":
    commArg = sys.argv
    Run(commArg)


